@@ -1,15 +1,19 @@
 use crate::data::position::Position;
 use crate::data::{
+    data::{init_child_context, init_child_scope, Data},
+    literal,
     literal::ContentType,
     primitive::{
         Primitive, PrimitiveBoolean, PrimitiveClosure, PrimitiveInt, PrimitiveNull,
         PrimitiveString, PrimitiveType, Right,
     },
     tokens::TYPES,
-    ArgsType, Data, Interval, Literal, Message, MessageData, MSG,
+    ArgsType, Interval, Literal, MemoryType, Message, MessageData, MSG,
 };
 use crate::error_format::*;
-use crate::interpreter::variable_handler::resolve_csml_object::exec_fn;
+use crate::interpreter::variable_handler::resolve_csml_object::{
+    exec_closure, insert_args_in_scope_memory, insert_memories_in_scope_memory,
+};
 use phf::phf_map;
 use rand::seq::SliceRandom;
 use rand::Rng;
@@ -26,6 +30,7 @@ use std::{collections::HashMap, sync::mpsc};
 type PrimitiveMethod = fn(
     array: &mut PrimitiveArray,
     args: &HashMap<String, Literal>,
+    additional_info: &Option<HashMap<String, Literal>>,
     interval: Interval,
     data: &mut Data,
     msg_data: &mut MessageData,
@@ -37,8 +42,11 @@ const FUNCTIONS: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map! {
     "is_int" => (PrimitiveArray::is_int as PrimitiveMethod, Right::Read),
     "is_float" => (PrimitiveArray::is_float as PrimitiveMethod, Right::Read),
     "type_of" => (PrimitiveArray::type_of as PrimitiveMethod, Right::Read),
+    "get_info" => (PrimitiveArray::get_info as PrimitiveMethod, Right::Read),
+    "is_error" => (PrimitiveArray::is_error as PrimitiveMethod, Right::Read),
     "to_string" => (PrimitiveArray::to_string as PrimitiveMethod, Right::Read),
 
+    "init" => (PrimitiveArray::init as PrimitiveMethod, Right::Read),
     "find" => (PrimitiveArray::find as PrimitiveMethod, Right::Read),
     "is_empty" => (PrimitiveArray::is_empty as PrimitiveMethod, Right::Read),
     "insert_at" => (PrimitiveArray::insert_at as PrimitiveMethod, Right::Write),
@@ -54,6 +62,9 @@ const FUNCTIONS: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map! {
     "map" => (PrimitiveArray::map as PrimitiveMethod, Right::Read),
     "filter" => (PrimitiveArray::filter as PrimitiveMethod, Right::Read),
     "reduce" => (PrimitiveArray::reduce as PrimitiveMethod, Right::Read),
+    "reverse" => (PrimitiveArray::reverse as PrimitiveMethod, Right::Read),
+    "append" => (PrimitiveArray::append as PrimitiveMethod, Right::Read),
+    "flatten" => (PrimitiveArray::flatten as PrimitiveMethod, Right::Read),
 };
 
 #[derive(PartialEq, Debug, Clone, Serialize, Deserialize)]
@@ -92,6 +103,7 @@ impl PrimitiveArray {
     fn is_number(
         _array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -112,6 +124,7 @@ impl PrimitiveArray {
     fn is_int(
         _array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -132,6 +145,7 @@ impl PrimitiveArray {
     fn is_float(
         _array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -152,6 +166,7 @@ impl PrimitiveArray {
     fn type_of(
         _array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -169,9 +184,39 @@ impl PrimitiveArray {
         Ok(PrimitiveString::get_literal("array", interval))
     }
 
+    fn get_info(
+        _array: &mut PrimitiveArray,
+        args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        literal::get_info(args, additional_info, interval, data)
+    }
+
+    fn is_error(
+        _array: &mut PrimitiveArray,
+        _args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        _data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        match additional_info {
+            Some(map) if map.contains_key("error") => {
+                Ok(PrimitiveBoolean::get_literal(true, interval))
+            }
+            _ => Ok(PrimitiveBoolean::get_literal(false, interval)),
+        }
+    }
+
     fn to_string(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -191,9 +236,53 @@ impl PrimitiveArray {
 }
 
 impl PrimitiveArray {
+    fn init(
+        _array: &mut PrimitiveArray,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "init(capacity: Int) => [Literal]";
+
+        if args.len() != 1 {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let capacity = match args.get("arg0") {
+            Some(res) if res.primitive.get_type() == PrimitiveType::PrimitiveInt => {
+                *Literal::get_value::<i64>(
+                    &res.primitive,
+                    &data.context.flow,
+                    interval,
+                    ERROR_ARRAY_INSERT_AT.to_owned(),
+                )? as usize
+            }
+            _ => {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    ERROR_ARRAY_INSERT_AT.to_owned(),
+                ));
+            }
+        };
+
+        let mut vec = Vec::with_capacity(capacity);
+        for _ in 0..capacity {
+            vec.push(PrimitiveNull::get_literal(interval));
+        }
+
+        Ok(PrimitiveArray::get_literal(&vec, interval))
+    }
+
     fn find(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -236,6 +325,7 @@ impl PrimitiveArray {
     fn is_empty(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -258,6 +348,7 @@ impl PrimitiveArray {
     fn insert_at(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -314,6 +405,7 @@ impl PrimitiveArray {
     fn index_of(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -350,6 +442,7 @@ impl PrimitiveArray {
     fn join(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -398,6 +491,7 @@ impl PrimitiveArray {
     fn length(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -420,6 +514,7 @@ impl PrimitiveArray {
     fn one_of(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -436,7 +531,7 @@ impl PrimitiveArray {
 
         if let Some(res) = array
             .value
-            .get(rand::thread_rng().gen_range(0, array.value.len()))
+            .get(rand::thread_rng().gen_range(0..array.value.len()))
         {
             return Ok(res.to_owned());
         }
@@ -447,6 +542,7 @@ impl PrimitiveArray {
     fn push(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -486,6 +582,7 @@ impl PrimitiveArray {
     fn pop(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -512,6 +609,7 @@ impl PrimitiveArray {
     fn remove_at(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -556,6 +654,7 @@ impl PrimitiveArray {
     fn shuffle(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -580,6 +679,7 @@ impl PrimitiveArray {
     fn slice(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         _msg_data: &mut MessageData,
@@ -690,12 +790,120 @@ impl PrimitiveArray {
             )),
         }
     }
+
+    fn reverse(
+        array: &mut PrimitiveArray,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "reverse() => [Literal]";
+
+        if !args.is_empty() {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let mut reversed_list = array.value.clone();
+        reversed_list.reverse();
+
+        Ok(PrimitiveArray::get_literal(&reversed_list, interval))
+    }
+
+    fn append(
+        array: &mut PrimitiveArray,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "append(other_array: [Literal]) => [Literal]";
+
+        if args.len() != 1 {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let mut other_array: Vec<Literal> = match args.get("arg0") {
+            Some(res) if res.content_type == "array" => {
+                let value = Literal::get_value::<Vec<Literal>>(
+                    &res.primitive,
+                    &data.context.flow,
+                    interval,
+                    ERROR_ARRAY_INSERT_AT.to_owned(),
+                )?;
+
+                (*value).clone().to_vec()
+            }
+            _ => {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    ERROR_ARRAY_INSERT_AT.to_owned(),
+                ));
+            }
+        };
+
+        let mut new_array = array.value.clone();
+
+        new_array.append(&mut other_array);
+
+        Ok(PrimitiveArray::get_literal(&new_array, interval))
+    }
+
+    fn flatten(
+        array: &mut PrimitiveArray,
+        args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
+        interval: Interval,
+        data: &mut Data,
+        _msg_data: &mut MessageData,
+        _sender: &Option<mpsc::Sender<MSG>>,
+    ) -> Result<Literal, ErrorInfo> {
+        let usage = "flatten() => [Literal]";
+
+        if args.len() != 0 {
+            return Err(gen_error_info(
+                Position::new(interval, &data.context.flow),
+                format!("usage: {}", usage),
+            ));
+        }
+
+        let mut new_array = vec![];
+
+        for lit in array.value.iter() {
+            if lit.content_type == "array" {
+                let value = Literal::get_value::<Vec<Literal>>(
+                    &lit.primitive,
+                    &data.context.flow,
+                    interval,
+                    ERROR_ARRAY_INSERT_AT.to_owned(),
+                )?;
+                for elem in value.iter() {
+                    new_array.push(elem.to_owned())
+                }
+            } else {
+                new_array.push(lit.to_owned())
+            }
+        }
+
+        Ok(PrimitiveArray::get_literal(&new_array, interval))
+    }
 }
 
 impl PrimitiveArray {
     fn map(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -713,6 +921,18 @@ impl PrimitiveArray {
                 )?;
 
                 let mut vec = vec![];
+                let mut context = init_child_context(&data);
+                let mut step_count = data.step_count.clone();
+                let mut new_scope_data = init_child_scope(data, &mut context, &mut step_count);
+
+                if let Some(memories) = closure.enclosed_variables.clone() {
+                    insert_memories_in_scope_memory(
+                        &mut new_scope_data,
+                        memories,
+                        msg_data,
+                        sender,
+                    );
+                }
 
                 for (index, value) in array.value.iter().enumerate() {
                     let mut map = HashMap::new();
@@ -725,13 +945,20 @@ impl PrimitiveArray {
                     }
 
                     let args = ArgsType::Normal(map);
-                    let result = exec_fn(
+                    insert_args_in_scope_memory(
+                        &mut new_scope_data,
+                        &closure.args,
+                        &args,
+                        msg_data,
+                        sender,
+                    );
+
+                    let result = exec_closure(
                         &closure.func,
                         &closure.args,
                         args,
-                        closure.enclosed_variables.clone(),
                         interval,
-                        data,
+                        &mut new_scope_data,
                         msg_data,
                         sender,
                     )?;
@@ -750,6 +977,7 @@ impl PrimitiveArray {
     fn filter(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -768,6 +996,19 @@ impl PrimitiveArray {
 
                 let mut vec = vec![];
 
+                let mut context = init_child_context(&data);
+                let mut step_count = data.step_count.clone();
+                let mut new_scope_data = init_child_scope(data, &mut context, &mut step_count);
+
+                if let Some(memories) = closure.enclosed_variables.clone() {
+                    insert_memories_in_scope_memory(
+                        &mut new_scope_data,
+                        memories,
+                        msg_data,
+                        sender,
+                    );
+                }
+
                 for (index, value) in array.value.iter().enumerate() {
                     let mut map = HashMap::new();
                     map.insert("arg0".to_owned(), value.to_owned());
@@ -780,16 +1021,24 @@ impl PrimitiveArray {
 
                     let args = ArgsType::Normal(map);
 
-                    let result = exec_fn(
+                    insert_args_in_scope_memory(
+                        &mut new_scope_data,
+                        &closure.args,
+                        &args,
+                        msg_data,
+                        sender,
+                    );
+
+                    let result = exec_closure(
                         &closure.func,
                         &closure.args,
                         args,
-                        closure.enclosed_variables.clone(),
                         interval,
-                        data,
+                        &mut new_scope_data,
                         msg_data,
                         sender,
                     )?;
+
                     if result.primitive.as_bool() {
                         vec.push(value.clone());
                     }
@@ -807,6 +1056,7 @@ impl PrimitiveArray {
     fn reduce(
         array: &mut PrimitiveArray,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         data: &mut Data,
         msg_data: &mut MessageData,
@@ -825,6 +1075,10 @@ impl PrimitiveArray {
                     format!("usage: {}", usage),
                 )?;
 
+                let mut context = init_child_context(&data);
+                let mut step_count = data.step_count.clone();
+                let mut new_scope_data = init_child_scope(data, &mut context, &mut step_count);
+
                 for (index, value) in array.value.iter().enumerate() {
                     let mut map = HashMap::new();
                     map.insert("arg0".to_owned(), accumulator);
@@ -839,13 +1093,20 @@ impl PrimitiveArray {
 
                     let args = ArgsType::Normal(map);
 
-                    accumulator = exec_fn(
+                    insert_args_in_scope_memory(
+                        &mut new_scope_data,
+                        &closure.args,
+                        &args,
+                        msg_data,
+                        sender,
+                    );
+
+                    accumulator = exec_closure(
                         &closure.func,
                         &closure.args,
                         args,
-                        closure.enclosed_variables.clone(),
                         interval,
-                        data,
+                        &mut new_scope_data,
                         msg_data,
                         sender,
                     )?;
@@ -878,6 +1139,8 @@ impl PrimitiveArray {
         Literal {
             content_type: "array".to_owned(),
             primitive,
+            additional_info: None,
+            secure_variable: false,
             interval,
         }
     }
@@ -1032,6 +1295,8 @@ impl Primitive for PrimitiveArray {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         _content_type: &ContentType,
         data: &mut Data,
@@ -1039,9 +1304,24 @@ impl Primitive for PrimitiveArray {
         sender: &Option<mpsc::Sender<MSG>>,
     ) -> Result<(Literal, Right), ErrorInfo> {
         if let Some((f, right)) = FUNCTIONS.get(name) {
-            let res = f(self, args, interval, data, msg_data, sender)?;
+            if *mem_type == MemoryType::Constant && *right == Right::Write {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("{}", ERROR_CONSTANT_MUTABLE_FUNCTION),
+                ));
+            } else {
+                let res = f(
+                    self,
+                    args,
+                    additional_info,
+                    interval,
+                    data,
+                    msg_data,
+                    sender,
+                )?;
 
-            return Ok((res, *right));
+                return Ok((res, *right));
+            }
         }
 
         Err(gen_error_info(
