@@ -1,5 +1,4 @@
 use crate::data::error_info::ErrorInfo;
-use crate::data::literal::ContentType;
 use crate::data::position::Position;
 use crate::data::primitive::boolean::PrimitiveBoolean;
 use crate::data::primitive::float::PrimitiveFloat;
@@ -8,7 +7,8 @@ use crate::data::primitive::string::PrimitiveString;
 use crate::data::primitive::tools::check_division_by_zero_i64;
 use crate::data::primitive::Right;
 use crate::data::primitive::{Primitive, PrimitiveType};
-use crate::data::{ast::Interval, message::Message, Data, Literal, MessageData, MSG};
+use crate::data::{ast::Interval, message::Message, Data, Literal, MemoryType, MessageData, MSG};
+use crate::data::{literal, literal::ContentType};
 use crate::error_format::*;
 use phf::phf_map;
 use serde::{Deserialize, Serialize};
@@ -22,6 +22,7 @@ use std::{collections::HashMap, sync::mpsc};
 type PrimitiveMethod = fn(
     int: &mut PrimitiveInt,
     args: &HashMap<String, Literal>,
+    additional_info: &Option<HashMap<String, Literal>>,
     data: &mut Data,
     interval: Interval,
 ) -> Result<Literal, ErrorInfo>;
@@ -31,6 +32,8 @@ const FUNCTIONS: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map! {
     "is_int" => (PrimitiveInt::is_int as PrimitiveMethod, Right::Read),
     "is_float" => (PrimitiveInt::is_float as PrimitiveMethod, Right::Read),
     "type_of" => (PrimitiveInt::type_of as PrimitiveMethod, Right::Read),
+    "is_error" => (PrimitiveInt::is_error as PrimitiveMethod, Right::Read),
+    "get_info" => (PrimitiveInt::get_info as PrimitiveMethod, Right::Read),
     "to_string" => (PrimitiveInt::to_string as PrimitiveMethod, Right::Read),
 
     "precision" => (PrimitiveInt::precision as PrimitiveMethod, Right::Read),
@@ -59,6 +62,7 @@ impl PrimitiveInt {
     fn is_number(
         _int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -77,6 +81,7 @@ impl PrimitiveInt {
     fn is_int(
         _int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -95,6 +100,7 @@ impl PrimitiveInt {
     fn is_float(
         _int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -113,6 +119,7 @@ impl PrimitiveInt {
     fn type_of(
         _int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -128,9 +135,35 @@ impl PrimitiveInt {
         Ok(PrimitiveString::get_literal("int", interval))
     }
 
+    fn get_info(
+        _int: &mut PrimitiveInt,
+        args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
+        data: &mut Data,
+        interval: Interval,
+    ) -> Result<Literal, ErrorInfo> {
+        literal::get_info(args, additional_info, interval, data)
+    }
+
+    fn is_error(
+        _int: &mut PrimitiveInt,
+        _args: &HashMap<String, Literal>,
+        additional_info: &Option<HashMap<String, Literal>>,
+        _data: &mut Data,
+        interval: Interval,
+    ) -> Result<Literal, ErrorInfo> {
+        match additional_info {
+            Some(map) if map.contains_key("error") => {
+                Ok(PrimitiveBoolean::get_literal(true, interval))
+            }
+            _ => Ok(PrimitiveBoolean::get_literal(false, interval)),
+        }
+    }
+
     fn to_string(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -151,6 +184,7 @@ impl PrimitiveInt {
     fn abs(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -174,6 +208,7 @@ impl PrimitiveInt {
     fn cos(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -199,6 +234,7 @@ impl PrimitiveInt {
     fn ceil(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -222,6 +258,7 @@ impl PrimitiveInt {
     fn precision(
         int: &mut PrimitiveInt,
         _args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         _data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -231,6 +268,7 @@ impl PrimitiveInt {
     fn floor(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -254,6 +292,7 @@ impl PrimitiveInt {
     fn pow(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -322,6 +361,7 @@ impl PrimitiveInt {
     fn round(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -345,6 +385,7 @@ impl PrimitiveInt {
     fn sin(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -370,6 +411,7 @@ impl PrimitiveInt {
     fn sqrt(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -395,6 +437,7 @@ impl PrimitiveInt {
     fn tan(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -420,6 +463,7 @@ impl PrimitiveInt {
     fn to_int(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -438,6 +482,7 @@ impl PrimitiveInt {
     fn to_float(
         int: &mut PrimitiveInt,
         args: &HashMap<String, Literal>,
+        _additional_info: &Option<HashMap<String, Literal>>,
         data: &mut Data,
         interval: Interval,
     ) -> Result<Literal, ErrorInfo> {
@@ -469,6 +514,8 @@ impl PrimitiveInt {
         Literal {
             content_type: "int".to_owned(),
             primitive,
+            additional_info: None,
+            secure_variable: false,
             interval,
         }
     }
@@ -649,6 +696,8 @@ impl Primitive for PrimitiveInt {
             Literal {
                 content_type: "int".to_owned(),
                 primitive: Box::new(PrimitiveString::new(&self.to_string())),
+                additional_info: None,
+                secure_variable: false,
                 interval: Interval {
                     start_column: 0,
                     start_line: 0,
@@ -681,6 +730,8 @@ impl Primitive for PrimitiveInt {
         &mut self,
         name: &str,
         args: &HashMap<String, Literal>,
+        mem_type: &MemoryType,
+        additional_info: &Option<HashMap<String, Literal>>,
         interval: Interval,
         _content_type: &ContentType,
         data: &mut Data,
@@ -688,9 +739,16 @@ impl Primitive for PrimitiveInt {
         _sender: &Option<mpsc::Sender<MSG>>,
     ) -> Result<(Literal, Right), ErrorInfo> {
         if let Some((f, right)) = FUNCTIONS.get(name) {
-            let res = f(self, args, data, interval)?;
+            if *mem_type == MemoryType::Constant && *right == Right::Write {
+                return Err(gen_error_info(
+                    Position::new(interval, &data.context.flow),
+                    format!("{}" , ERROR_CONSTANT_MUTABLE_FUNCTION),
+                ));
+            } else {
+                let res = f(self, args, additional_info, data, interval)?;
 
-            return Ok((res, *right));
+                return Ok((res, *right));
+            }
         }
 
         Err(gen_error_info(
