@@ -51,8 +51,8 @@ const FUNCTIONS: phf::Map<&'static str, (PrimitiveMethod, Right)> = phf_map! {
     "decode_uri" => (PrimitiveString::decode_uri as PrimitiveMethod, Right::Read),
     "encode_uri_component" => (PrimitiveString::encode_uri_component as PrimitiveMethod, Right::Read),
     "decode_uri_component" => (PrimitiveString::decode_uri_component as PrimitiveMethod, Right::Read),
-    "encode_html_escape" => (PrimitiveString::encode_html_escape as PrimitiveMethod, Right::Read),
-    "decode_html_escape" => (PrimitiveString::decode_html_escape as PrimitiveMethod, Right::Read),
+    "encode_html_entities" => (PrimitiveString::encode_html_entities as PrimitiveMethod, Right::Read),
+    "decode_html_entities" => (PrimitiveString::decode_html_entities as PrimitiveMethod, Right::Read),
 
     "is_email" => (PrimitiveString::is_email as PrimitiveMethod, Right::Read),
     "append" => (PrimitiveString::append as PrimitiveMethod, Right::Read),
@@ -418,11 +418,11 @@ impl PrimitiveString {
 
         let vec: Vec<&str> = string.value.split('?').collect();
 
-        let (q_serparator, query, separator, fragment) = if vec.len() > 1 {
+        let (q_separator, query, separator, fragment) = if vec.len() > 1 {
             let url = Url::parse(&string.value).unwrap();
 
             let query_pairs = url.query_pairs();
-            let (q_serparator, query) = match query_pairs.count() {
+            let (q_separator, query) = match query_pairs.count() {
                 0 => ("", "".to_owned()),
                 _ => {
                     let query = encode_value(&query_pairs);
@@ -440,13 +440,13 @@ impl PrimitiveString {
                 None => ("", "".to_owned()),
             };
 
-            (q_serparator, query, f_serparatorm, fragment)
+            (q_separator, query, f_serparatorm, fragment)
         } else {
             ("", "".to_owned(), "", "".to_owned())
         };
 
         Ok(PrimitiveString::get_literal(
-            &format!("{}{q_serparator}{query}{separator}{fragment}", vec[0]),
+            &format!("{}{q_separator}{query}{separator}{fragment}", vec[0]),
             interval,
         ))
     }
@@ -471,11 +471,11 @@ impl PrimitiveString {
 
         let vec: Vec<&str> = string.value.split('?').collect();
 
-        let (q_serparator, query, separator, fragment) = if vec.len() > 1 {
+        let (q_separator, query, separator, fragment) = if vec.len() > 1 {
             let url = Url::parse(&string.value).unwrap();
 
             let query_pairs = url.query_pairs();
-            let (q_serparator, query) = match query_pairs.count() {
+            let (q_separator, query) = match query_pairs.count() {
                 0 => ("", "".to_owned()),
                 _ => {
                     let query = decode_value(&query_pairs);
@@ -493,13 +493,13 @@ impl PrimitiveString {
                 None => ("", "".to_owned()),
             };
 
-            (q_serparator, query, f_serparatorm, fragment)
+            (q_separator, query, f_serparatorm, fragment)
         } else {
             ("", "".to_owned(), "", "".to_owned())
         };
 
         Ok(PrimitiveString::get_literal(
-            &format!("{}{q_serparator}{query}{separator}{fragment}", vec[0]),
+            &format!("{}{q_separator}{query}{separator}{fragment}", vec[0]),
             interval,
         ))
     }
@@ -557,7 +557,7 @@ impl PrimitiveString {
         }
     }
 
-    fn decode_html_escape(
+    fn decode_html_entities(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
         _additional_info: &Option<HashMap<String, Literal>>,
@@ -566,7 +566,7 @@ impl PrimitiveString {
         _msg_data: &mut MessageData,
         _sender: &Option<mpsc::Sender<MSG>>,
     ) -> Result<Literal, ErrorInfo> {
-        let usage = "decode_html_escape() => String";
+        let usage = "decode_html_entities() => String";
 
         if !args.is_empty() {
             return Err(gen_error_info(
@@ -580,7 +580,7 @@ impl PrimitiveString {
         Ok(PrimitiveString::get_literal(&decoded, interval))
     }
 
-    fn encode_html_escape(
+    fn encode_html_entities(
         string: &mut PrimitiveString,
         args: &HashMap<String, Literal>,
         _additional_info: &Option<HashMap<String, Literal>>,
@@ -589,7 +589,7 @@ impl PrimitiveString {
         _msg_data: &mut MessageData,
         _sender: &Option<mpsc::Sender<MSG>>,
     ) -> Result<Literal, ErrorInfo> {
-        let usage = "encode_html_escape() => String";
+        let usage = "encode_html_entities() => String";
 
         if !args.is_empty() {
             return Err(gen_error_info(
