@@ -1,12 +1,12 @@
 pub mod api;
 pub mod crypto;
+pub mod exists;
 pub mod format;
 pub mod functions;
-pub mod http;
-pub mod smtp;
+pub mod http_builtin;
 pub mod jwt;
+pub mod smtp;
 pub mod time;
-pub mod exists;
 
 pub mod tools;
 
@@ -19,13 +19,14 @@ use std::sync::mpsc;
 
 use api::api;
 use crypto::crypto;
+use exists::exists;
 use format::*;
 use functions::*;
-use http::http;
-use smtp::smtp;
+use http_builtin::http;
 use jwt::jwt;
+use smtp::smtp;
 use time::time;
-use exists::exists;
+// use uri::*;
 
 pub fn match_native_builtin(
     name: &str,
@@ -58,6 +59,7 @@ pub fn match_builtin(
         HEX => hex(args, &data.context.flow, interval),
         FN | APP => api(args, interval, data, msg_data, sender),
         ONE_OF => one_of(args, &data.context.flow, interval),
+        OR_BUILT_IN => or(args, &data.context.flow, interval),
         SHUFFLE => shuffle(args, &data.context.flow, interval),
         LENGTH => length(args, &data.context.flow, interval),
         FIND => find(args, &data.context.flow, interval),
