@@ -1,16 +1,16 @@
 use crate::data::error_info::ErrorInfo;
 use crate::data::literal::ContentType;
 use crate::data::primitive::{closure::capture_variables, PrimitiveArray, PrimitiveObject};
-use crate::data::{Position, warnings::DisplayWarnings};
-use crate::data::{ast::*, ArgsType, Data, Literal, MessageData, MSG};
+use crate::data::{
+    ast::*, warnings::DisplayWarnings, ArgsType, Data, Literal, MemoryType, MessageData, Position,
+    MSG,
+};
 use crate::error_format::*;
 use crate::interpreter::{
     ast_interpreter::evaluate_condition,
-    json_to_rust::interpolate,
     variable_handler::{
         exec_path_actions, get_string_from_complex_string, get_var, interval::interval_from_expr,
-        resolve_csml_object::resolve_object, resolve_path,
-        operations::{evaluate_postfix},
+        operations::evaluate_postfix, resolve_csml_object::resolve_object, resolve_path,
     },
 };
 use std::{collections::HashMap, sync::mpsc};
@@ -29,9 +29,10 @@ fn exec_path_literal(
 ) -> Result<Literal, ErrorInfo> {
     if let Some(path) = path {
         let path = resolve_path(path, dis_warnings, data, msg_data, sender)?;
-        let (mut new_literal, ..) = exec_path_actions(
+        let (new_literal, ..) = exec_path_actions(
             literal,
             dis_warnings,
+            &MemoryType::Use,
             None,
             &Some(path),
             &ContentType::get(&literal),
@@ -40,11 +41,6 @@ fn exec_path_literal(
             sender,
         )?;
 
-        if new_literal.content_type == "string" {
-            let string = serde_json::json!(new_literal.primitive.to_string());
-            new_literal = interpolate(&string, new_literal.interval, data, msg_data, sender)?;
-        }
-
         Ok(new_literal)
     } else {
         Ok(literal.to_owned())
@@ -87,14 +83,20 @@ pub fn expr_to_literal(
             ..
         } => {
             let mut map = HashMap::new();
+            let mut is_secure = false;
 
             for (key, value) in object.iter() {
-                map.insert(
-                    key.to_owned(),
-                    expr_to_literal(&value, dis_warnings, None, data, msg_data, sender)?,
-                );
+                let lit = expr_to_literal(&value, dis_warnings, None, data, msg_data, sender)?;
+                if lit.secure_variable {
+                    is_secure = true;
+                }
+
+                map.insert(key.to_owned(), lit);
             }
+
             let mut literal = PrimitiveObject::get_literal(&map, range_interval.to_owned());
+            literal.secure_variable = is_secure;
+
             exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
         Expr::ComplexLiteral(vec, range_interval) => {
@@ -109,16 +111,23 @@ pub fn expr_to_literal(
         }
         Expr::VecExpr(vec, range_interval) => {
             let mut array = vec![];
+            let mut is_secure = false;
+
             for value in vec.iter() {
-                array.push(expr_to_literal(
-                    value, dis_warnings, None, data, msg_data, sender,
-                )?)
+                let lit = expr_to_literal(value, dis_warnings, None, data, msg_data, sender)?;
+                if lit.secure_variable {
+                    is_secure = true;
+                }
+
+                array.push(lit)
             }
             let mut literal = PrimitiveArray::get_literal(&array, range_interval.to_owned());
+            literal.secure_variable = is_secure;
+
             exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
-        Expr::PostfixExpr(postfix, expr) => {
-            let mut literal = evaluate_postfix(postfix, expr, data, msg_data, sender)?;
+        Expr::PostfixExpr(pretfix, expr) => {
+            let mut literal = evaluate_postfix(pretfix, expr, data, msg_data, sender)?;
             exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
         Expr::InfixExpr(infix, exp_1, exp_2) => {
@@ -181,7 +190,8 @@ pub fn resolve_fn_args(
                         };
                         named_args = true;
 
-                        let literal = expr_to_literal(var, dis_warnings, None, data, msg_data, sender)?;
+                        let literal =
+                            expr_to_literal(var, dis_warnings, None, data, msg_data, sender)?;
                         map.insert(name.ident.to_owned(), literal);
                     }
                     expr => {
@@ -189,10 +199,11 @@ pub fn resolve_fn_args(
                         if named_args && first > 1 {
                             return Err(gen_error_info(
                                 Position::new(interval_from_expr(expr), &data.context.flow),
-                                ERROR_EXPR_TO_LITERAL.to_owned(), // TODO: error mix of named args and anonymous args
+                                ERROR_EXPR_TO_LITERAL.to_owned(),
                             ));
                         }
-                        let literal = expr_to_literal(expr, dis_warnings, None, data, msg_data, sender)?;
+                        let literal =
+                            expr_to_literal(expr, dis_warnings, None, data, msg_data, sender)?;
                         map.insert(format!("arg{}", index), literal);
                     }
                 }
@@ -205,7 +216,7 @@ pub fn resolve_fn_args(
         }
         e => Err(gen_error_info(
             Position::new(interval_from_expr(e), &data.context.flow),
-            ERROR_EXPR_TO_LITERAL.to_owned(), //TODO: internal error fn args bad format
+            ERROR_EXPR_TO_LITERAL.to_owned(),
         )),
     }
 }
