@@ -3,10 +3,11 @@ use crate::data::position::Position;
 use crate::data::{
     ast::PathLiteral,
     primitive::{PrimitiveNull, PrimitiveString},
+    warnings::DisplayWarnings,
 };
 use crate::data::{
     ast::{Interval, PathState},
-    Data, Literal, MessageData, MSG,
+    Data, Literal, MemoryType, MessageData, MSG,
 };
 use crate::error_format::*;
 use crate::interpreter::variable_handler::gen_generic_component::gen_generic_component;
@@ -22,7 +23,7 @@ use std::sync::mpsc;
 
 pub fn gen_literal_from_event(
     interval: Interval,
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
     path: Option<&[(Interval, PathState)]>,
     data: &mut Data,
     msg_data: &mut MessageData,
@@ -30,8 +31,9 @@ pub fn gen_literal_from_event(
 ) -> Result<Literal, ErrorInfo> {
     match path {
         Some(path) => {
-            let path = resolve_path(path, condition, data, msg_data, sender)?;
-            let mut lit = json_to_literal(&data.event.content, interval.to_owned(), &data.context.flow)?;
+            let path = resolve_path(path, dis_warnings, data, msg_data, sender)?;
+            let mut lit =
+                json_to_literal(&data.event.content, interval.to_owned(), &data.context.flow)?;
 
             lit.set_content_type("event");
 
@@ -47,7 +49,8 @@ pub fn gen_literal_from_event(
 
             let (lit, _tmp_mem_update) = exec_path_actions(
                 &mut lit,
-                condition,
+                dis_warnings,
+                &MemoryType::Event("event".to_owned()),
                 None,
                 &Some(path),
                 &content_type,
@@ -58,10 +61,14 @@ pub fn gen_literal_from_event(
 
             Ok(lit)
         }
-        None => Ok(PrimitiveString::get_literal(
-            &data.event.content_value,
-            interval.to_owned(),
-        )),
+        None => {
+            let mut lit =
+                PrimitiveString::get_literal(&data.event.content_value, interval.to_owned());
+
+            lit.secure_variable = data.event.secure;
+
+            Ok(lit)
+        }
     }
 }
 
@@ -74,7 +81,7 @@ pub fn gen_literal_from_component(
 ) -> Result<Literal, ErrorInfo> {
     match path {
         Some(path) => {
-            let mut path = resolve_path(path, false, data, msg_data, sender)?;
+            let mut path = resolve_path(path, &DisplayWarnings::On, data, msg_data, sender)?;
 
             if let Some((_interval, function_name)) = path.first() {
                 if let PathLiteral::Func {
@@ -84,13 +91,21 @@ pub fn gen_literal_from_component(
                 } = function_name
                 {
                     if let Some(component) = data.custom_component.get(name) {
-                        let mut lit = gen_generic_component(name, true, &data.context.flow, interval, args, component)?;
+                        let mut lit = gen_generic_component(
+                            name,
+                            true,
+                            &data.context.flow,
+                            interval,
+                            args,
+                            component,
+                        )?;
 
                         path.drain(..1);
 
                         let (lit, _tmp_mem_update) = exec_path_actions(
                             &mut lit,
-                            false,
+                            &DisplayWarnings::On,
+                            &MemoryType::Use,
                             None,
                             &Some(path),
                             &ContentType::Primitive,
@@ -118,7 +133,7 @@ pub fn gen_literal_from_component(
 
 pub fn get_literal_from_metadata(
     path: &[(Interval, PathLiteral)],
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
     data: &mut Data,
     msg_data: &mut MessageData,
     interval: Interval,
@@ -140,7 +155,8 @@ pub fn get_literal_from_metadata(
     let content_type = ContentType::get(&lit);
     let (lit, _tmp_mem_update) = exec_path_actions(
         &mut lit,
-        condition,
+        dis_warnings,
+        &MemoryType::Metadata,
         None,
         &Some(path[1..].to_owned()),
         &content_type,
